/*
 * window_main.c
 * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * wmlogut is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * simple-netaid is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the COPYING file. * 
 */

#include "config.h"
#include "window_main.h"
#include "buttonbox.h"

#include <err.h>
#include <libintl.h>
#include <locale.h>

#include <gdk/gdkscreen.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <cairo.h>

#define _(X) gettext(X)

/* Only some X servers support alpha channels. Always have a fallback */
gboolean supports_alpha = FALSE;


WindowMain::WindowMain(Config* params)
{
	GtkWidget *_gobj = GTK_WIDGET(this->gobj());
	//gtk_window_set_title(GTK_WINDOW(_gobj), "Logout dialog");	
	
	set_title(_("Logout dialog"));
	set_default_size(310, 200);
	set_position(Gtk::WIN_POS_CENTER);
    g_signal_connect(G_OBJECT(_gobj), "delete-event", gtk_main_quit, NULL);

    /* Tell GTK+ that we want to draw the windows background ourself.
     * If we don't do this then GTK+ will clear the window to the
     * opaque theme default color, which isn't what we want.
     */
	gtk_widget_set_app_paintable(_gobj, true);

    /* We need to handle two events ourself: "expose-event" and "screen-changed".
     *
     * The X server sends us an expose event when the window becomes
     * visible on screen. It means we need to draw the contents.  On a
     * composited desktop expose is normally only sent when the window
     * is put on the screen. On a non-composited desktop it can be
     * sent whenever the window is uncovered by another.
     *
     * The screen-changed event means the display to which we are
     * drawing changed. GTK+ supports migration of running
     * applications between X servers, which might not support the
     * same features, so we need to check each time.
     */
    g_signal_connect(G_OBJECT(_gobj), "expose-event", G_CALLBACK(expose), NULL);
    g_signal_connect(G_OBJECT(_gobj), "screen-changed", G_CALLBACK(screen_changed), NULL);

    /* toggle title bar on click - we add the mask to tell X we are interested in this event */
    gtk_window_set_decorated(GTK_WINDOW(_gobj), FALSE);
    gtk_widget_add_events(_gobj, GDK_BUTTON_PRESS_MASK);
    g_signal_connect(G_OBJECT(_gobj), "button-press-event", G_CALLBACK(clicked), NULL);

    /* initialize for the current display */
    screen_changed(_gobj, NULL, NULL);
    
	add(m_HBox);
	m_HBox.add(m_Align1);
    Gtk::HBox *hBox1 = new Gtk::HBox;
    m_Align1.add(*hBox1);

    hBox1->set_border_width(10);
	
	hBox1->add(m_Image);
	
	m_Image.set(params->LOGO);

	m_HBox.add(m_Align2);
    Gtk::HBox *hBox2 = new Gtk::HBox;
    m_Align2.add(*hBox2);

    hBox2->set_border_width(10);
 
    myButtonBox *pButtonBox = new myButtonBox(false, 10, Gtk::BUTTONBOX_END);
    hBox2->pack_start(*pButtonBox, true, true, 5);
    
    //m_ShutdownButton->use_stock(true);
    m_ShutdownButton = Gtk::manage(new Gtk::Button());
    pButtonBox->bbox->add(*m_ShutdownButton);
    m_ShutdownButton->add_pixlabel(params->SHUTDOWN_ICON, _("Shutdown")); 
	m_ShutdownButton->signal_clicked().connect
	(
		sigc::bind<Config*>
		(
			sigc::mem_fun(*this, &WindowMain::on_shutdown_clicked),
			params
		)
	);
    
    m_CancelButton = Gtk::manage(new Gtk::Button());
    pButtonBox->bbox->add(*m_CancelButton);
    m_CancelButton->add_pixlabel(params->CANCEL_ICON, _("Cancel"));
	m_CancelButton->signal_clicked().connect
	(
		sigc::mem_fun
		(
			*this,
			&WindowMain::on_cancel_clicked
		)
	);
    
    m_RebootButton = Gtk::manage(new Gtk::Button());
    pButtonBox->bbox->add(*m_RebootButton);
    m_RebootButton->add_pixlabel(params->REBOOT_ICON, _("Reboot")); 
	m_RebootButton->signal_clicked().connect
	(
		sigc::bind<Config*>
		(
			sigc::mem_fun(*this, &WindowMain::on_reboot_clicked),
			params
		)
	);
    
    m_LogoutButton = Gtk::manage(new Gtk::Button());
    pButtonBox->bbox->add(*m_LogoutButton);
    m_LogoutButton->add_pixlabel(params->LOGOUT_ICON, _("Logout"));  
	m_LogoutButton->signal_clicked().connect
	(
		sigc::bind<Config*>
		(
			sigc::mem_fun(*this, &WindowMain::on_logout_clicked),
			params
		)
	);

	show_all_children();

	// The widget must be realized and mapped before grab_focus() is called.
	// That's why it's called after show_all_children().
	m_ShutdownButton->grab_focus();
}

WindowMain::~WindowMain()
{

}

void WindowMain::on_shutdown_clicked(Config* params)
{
	system(params->SHUTDOWN_CMD.c_str());
	hide();
}

void WindowMain::on_cancel_clicked()
{
	hide();
}

void WindowMain::on_reboot_clicked(Config* params)
{
	system(params->REBOOT_CMD.c_str());
	hide();
}

void WindowMain::on_logout_clicked(Config* params)
{
	system("openbox --exit");
	hide();
}


/* *
 * 
 *   Gtk+
 *
 */

void screen_changed(GtkWidget *widget, GdkScreen *old_screen, gpointer userdata)
{
	   /* To check if the display supports alpha channels, get the colormap */
    GdkScreen *screen = gtk_widget_get_screen(widget);
    GdkColormap *colormap = gdk_screen_get_rgba_colormap(screen);

    if (!colormap) /* the screen does not support alpha channels */
    {
        colormap = gdk_screen_get_rgb_colormap(screen);
        supports_alpha = false;
    }
    else  /* the screen supports alpha channels */
    {
        supports_alpha = true;
    }

    /* Now we have a colormap appropriate for the screen, use it */
    gtk_widget_set_colormap(widget, colormap);
}

/* This is called when we need to draw the windows contents */
gboolean expose(GtkWidget *widget, GdkEventExpose *event, gpointer userdata)
{
    cairo_t *cr = gdk_cairo_create(widget->window);
    
    if (supports_alpha)
        cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 0.75); // transparency
    else
        cairo_set_source_rgb (cr, 1.0, 1.0, 1.0); // opaque white

    /* draw the background */
    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
    //cairo_paint (cr);

    /* draw a circle */
    int width, height;
    gtk_window_get_size(GTK_WINDOW(widget), &width, &height);
    
   /*
    // load image and get dimantions
    cairo_surface_t *img;
    int imgw, imgh;
    char *imgpath;

    imgpath = "/home/aitor/LOGOUT/simple-netaid-gtk/logo/gnu.png";
    if (imgpath == NULL) {
        fprintf(stderr, "usage: see image.png\n");
        exit(1);
    }
 
    img = cairo_image_surface_create_from_png(imgpath);
    if (cairo_surface_status(img) != CAIRO_STATUS_SUCCESS)
        errx(1, "load image");
    imgw = cairo_image_surface_get_width(img);
    imgh = cairo_image_surface_get_height(img);

    gtk_window_resize(GTK_WINDOW(widget), imgw, imgh);

    cairo_set_source_surface(cr, img, 0, 0);
  */
  
    cairo_paint (cr);

    cairo_destroy(cr);
    return FALSE;
}

void clicked(GtkWindow *win, GdkEventButton *event, gpointer user_data)
{
    /* toggle window manager frames */
    gtk_window_set_decorated(win, !gtk_window_get_decorated(win));
}


